package com.college.data.repositorty.role

import com.college.data.DbSettings
import com.college.data.mapper.resultRowToRole
import com.college.data.table.RoleIdTable
import com.college.domain.model.Role
import com.college.domain.request.role.RoleCreateRequest
import com.college.domain.request.role.RoleUpdateRequest
import org.jetbrains.exposed.sql.SqlExpressionBuilder.eq
import org.jetbrains.exposed.sql.deleteWhere
import org.jetbrains.exposed.sql.insertReturning
import org.jetbrains.exposed.sql.selectAll
import org.jetbrains.exposed.sql.update

class RoleRepositoryImpl(private val dbSettings: DbSettings): RoleRepository {
    override suspend fun getAllRoles(): List<Role> = dbSettings.query {
        RoleIdTable.selectAll().map(::resultRowToRole)
    }

    override suspend fun createRole(roleCreateRequest: RoleCreateRequest): Role = dbSettings.query {
        val result = RoleIdTable.insertReturning {
            it[role] = roleCreateRequest.roleName
        }
        result.map(::resultRowToRole).first()
    }

    override suspend fun removeRoleById(rolId: Int): Boolean  = dbSettings.query {
        val result = RoleIdTable.deleteWhere {
            RoleIdTable.id eq rolId
        }
        result > 0
    }

    override suspend fun updateRole(rolId: Int, roleUpdateRequest: RoleUpdateRequest): Boolean = dbSettings.query {
        val result = RoleIdTable.update({RoleIdTable.id eq rolId}){
            it[role] = roleUpdateRequest.roleName
        }
        result > 0
    }
}