package com.college.data.repositorty.user

import com.college.data.DbSettings
import com.college.data.mapper.resultRowToUser
import com.college.data.mapper.resultRowToUserWithJoinRole
import com.college.data.table.RoleIdTable
import com.college.data.table.UserUuidTable
import com.college.domain.model.User
import com.college.domain.request.user.UpdateUserRequest
import com.college.domain.request.user.UserCreateRequest
import org.jetbrains.exposed.sql.SqlExpressionBuilder.eq
import org.jetbrains.exposed.sql.deleteWhere
import org.jetbrains.exposed.sql.insertReturning
import org.jetbrains.exposed.sql.selectAll
import org.jetbrains.exposed.sql.update
import java.util.*

class UserRepositoryImpl(private val database: DbSettings): UserRepository {
    override suspend fun getUserByUuid(userUuid: UUID): User? = database.query {
        (UserUuidTable innerJoin RoleIdTable).selectAll().where {
            UserUuidTable.id eq userUuid
        }
            .map(::resultRowToUserWithJoinRole)
            .firstOrNull()
    }

    override suspend fun removeUserByUuid(userUuid: UUID): Boolean = database.query{
        val result = UserUuidTable.deleteWhere {
            UserUuidTable.id eq userUuid
        }
        result > 0
    }

    override suspend fun addUser(userCreateRequest: UserCreateRequest): User = database.query {
      val result =   UserUuidTable.insertReturning {
            it[password] = userCreateRequest.password
            it[login] = userCreateRequest.login
            it[role] = userCreateRequest.role.id
        }
        resultRowToUser(result.first())
    }

    override suspend fun getAllUsers(): List<User> = database.query{
        (UserUuidTable innerJoin RoleIdTable)
            .selectAll()
            .map(::resultRowToUserWithJoinRole)
    }

    override suspend fun updateUser(userUuid: UUID, updateUserRequest: UpdateUserRequest): Boolean  = database.query{
        val result = UserUuidTable.update({UserUuidTable.id eq userUuid}){ oldUser ->
            oldUser[login] = updateUserRequest.login
            oldUser[password] = updateUserRequest.password
            oldUser[isApproved] = updateUserRequest.isApproved
            val newRole = checkNotNull(updateUserRequest.role)
            oldUser[role] = newRole.id
        }
        result > 0
    }

}